use crate::handlers::markups::deletion_markup;
use arrayvec::{ArrayString, ArrayVec};
use rand::{rngs::OsRng, seq::SliceRandom};
use std::str::from_utf8_unchecked;
use teloxide::{adaptors::Throttle, prelude::*, types::ParseMode};
use tokio::task::spawn_blocking;

const CHARS: &[u8] = br##"!"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[]^_abcdefghijklmnopqrstuvwxyz{|}~"##;

/// Returns true if the generated master password is valid.
/// It checks that it has at least one lowercase, one lowercase and one punctuation char
#[inline]
fn check_generated_password(password: &[u8]) -> bool {
    let mut flags: u8 = 0;
    for &byte in password {
        match byte {
            b'a'..=b'z' => flags |= 0b1,
            b'A'..=b'Z' => flags |= 0b10,
            b'0'..=b'9' => flags |= 0b100,
            b'!'..=b'/' | b':'..=b'@' | b'['..=b'`' | b'{'..=b'~' => flags |= 0b1000,
            _ => (),
        }
        if flags == 0b1111 {
            return true;
        }
    }
    false
}

/// Continuously generates the password until it passes the checks
#[inline]
fn generete_passwords() -> ArrayVec<ArrayString<34>, 10> {
    let mut passwords = ArrayVec::new_const();
    while !passwords.is_full() {
        let password: ArrayVec<u8, 32> = (0..32)
            .map(|_| *CHARS.choose(&mut OsRng).unwrap())
            .collect();
        if check_generated_password(&password) {
            let mut string = ArrayString::<34>::new_const();
            string.push('`');
            unsafe { string.push_str(from_utf8_unchecked(&password)) };
            string.push('`');
            passwords.push(string)
        }
    }
    passwords
}

/// Handles /gen_password command by generating 10 copyable passwords and sending them to the user
pub async fn gen_password(bot: Throttle<Bot>, msg: Message) -> crate::Result<()> {
    let mut message: ArrayString<{ 10 + 35 * 10 }> = "Passwords:".try_into().unwrap();
    let passwords = spawn_blocking(generete_passwords).await?;
    for password in passwords {
        message.push('\n');
        message.push_str(&password)
    }
    bot.send_message(msg.chat.id, message.as_str())
        .parse_mode(ParseMode::MarkdownV2)
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}
