use crate::{
    entity::prelude::MasterPass,
    errors::NoUserInfo,
    handlers::{markups::deletion_markup, MainDialogue},
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*};
use tokio::task::spawn_blocking;

/// Returns true if the provided master password is valid
#[inline]
pub async fn check_master_pass<'a>(
    bot: &'a Throttle<Bot>,
    msg: &'a Message,
    db: &'a DatabaseConnection,
    master_pass: &'a str,
) -> crate::Result<bool> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let model = MasterPass::get(user_id, db).await?;
    let is_valid = match model {
        Some(model) => {
            let master_pass = master_pass.to_owned();
            spawn_blocking(move || model.verify(&master_pass)).await??
        }
        None => {
            bot.send_message(msg.chat.id, "No master password set")
                .reply_markup(deletion_markup())
                .await?;
            return Ok(false);
        }
    };
    if !is_valid {
        bot.send_message(msg.chat.id, "Wrong master password")
            .reply_markup(deletion_markup())
            .await?;
    }
    Ok(is_valid)
}

pub async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    next: super::PackagedHandler<String>,
) -> crate::Result<()> {
    super::generic::generic(
        bot,
        msg,
        db,
        dialogue,
        |bot, msg, db, text| Box::pin(check_master_pass(bot, msg, db, text)),
        next,
    )
    .await
}
