use crate::handlers::MainDialogue;
use sea_orm::prelude::*;
use std::{future::Future, sync::Arc};
use teloxide::{adaptors::Throttle, prelude::*};
use tokio::sync::Mutex;

#[allow(clippy::type_complexity)]
pub struct Handler<T> {
    pub handler: Box<
        dyn FnOnce(
                Throttle<Bot>,
                Message,
                DatabaseConnection,
                MainDialogue,
                T,
            ) -> crate::PinnedFuture<'static, crate::Result<()>>
            + Send
            + Sync,
    >,

    pub previous: Option<Message>,
}

pub type PackagedHandler<T> = Arc<Mutex<Option<Handler<T>>>>;

impl<T> Handler<T> {
    /// Convinience method to convert a simple async function and a previous message into PackagedHandler
    #[inline]
    pub fn new<H, F>(f: H, previous: impl Into<Option<Message>>) -> PackagedHandler<T>
    where
        H: FnOnce(Throttle<Bot>, Message, DatabaseConnection, MainDialogue, T) -> F
            + Send
            + Sync
            + 'static,
        F: Future<Output = crate::Result<()>> + Send + 'static,
    {
        let handler = Self {
            handler: Box::new(move |bot, msg, db, dialogue, val| {
                Box::pin(f(bot, msg, db, dialogue, val))
            }),
            previous: previous.into(),
        };
        Arc::new(Mutex::new(Some(handler)))
    }
}
